# -*- coding: utf-8 -*-
import re, sys, os

def getTestSets(package):
  """
  Get test sets from files in ./test_sets/
  """
  if '/' in package:
    package_path, package_name = package.rsplit('/', 1)
  else:
    package_path = '.'
    package_name = package
  sys.path.append(os.path.realpath(package_path))

  try:
    __package__ = __import__(package_name)
  except:
    print 'No such package.'
    sys.exit(1)

  test_sets = []
  for mod_name in dir(__package__):
    __module__ = getattr(__package__, mod_name)
    for class_name in dir(__module__):
      if re.match('Set*', class_name):
        obj = getattr(__module__, class_name)
        test_sets.append(obj())

  return test_sets



# -*- coding: utf-8 -*-

# Python imports
import re, requests

# Local imports
import settings
import printers

class TestSet:
  """
  All TestSets must inherit from this class
  """
  _base_url = ""

  def setUp(self):
    """
    Called before the beginning of the test set
    """
    return

  def setDown(self):
    """
    Called after the end of the test set
    """
    return

  def get(self, end_url, **kwargs):
    url = self._base_url + end_url
    return requests.get(url, **kwargs)

  def post(self, end_url, data=None, json=None, **kwargs):
    url = self._base_url + end_url
    return requests.post(url, data, json, **kwargs)

  def put(self, end_url, data=None, **kwargs):
    url = self._base_url + end_url
    return requests.put(url, data, **kwargs)

  def patch(self, end_url, data=None, **kwargs):
    url = self._base_url + end_url
    return requests.patch(url, data, **kwargs)

  def delete(self, end_url, **kwargs):
    url = self._base_url + end_url
    return requests.delete(url, **kwargs)

  def expect(self, response, code=None, body=None):
    """
    Return whether the response corresponds to what is expected or not
    """
    success = True

    if code != None and code != response.status_code:
      success = False

    if body != None and body != response.body:
      success = False
      #print body

    return {'success': success, 'code': response.status_code, 'elapsed': response.elapsed}

class App:
  """
  Main entry
  """
  def __init__(self, printer):
    self.printer = printer

  def process(self, modules):
    """
    Process modules
    """
    data_total = {}
    data_total['index'] = 0
    data_total['nb_ok'] = 0

    if len(modules) == 0:
      self.printer.printErrorNoSetFound()
      return

    self.printer.printIntro()

    for test_set in modules:

      self.printer.printSetIntro(test_set)

      data_test_set = {}
      data_test_set['index'] = 0
      data_test_set['nb_ok'] = 0

      test_set.setUp()

      for f in dir(test_set):
        if re.match('test_*', f):
          data_test_set['index'] += 1
          func = getattr(test_set, f)
          func_doc = func.__doc__.strip('\n')

          try:
            data_test = func()

            if data_test['success']:
              data_test_set['nb_ok'] += 1


            self.printer.printTestOutput(data_test, func_doc)
          except Exception as e:
            self.printer.printTestDirtyFailure({'success': False, 'exception': e})


      test_set.setDown()

      data_total['index'] += data_test_set['index']
      data_total['nb_ok'] += data_test_set['nb_ok']
      self.printer.printSetResult(test_set, data_test_set['index'], data_test_set['nb_ok'], 0)


    self.printer.printTotalResult(data_total['index'], data_total['nb_ok'], 0)
    return 0 if data_total['index'] == data_total['nb_ok'] else 1



# -*- coding: utf-8 -*-

# Python import
import sys

# Local import
import settings

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

#following from Python cookbook, #475186
def has_colors(stream):
  if not hasattr(stream, "isatty") or not stream.isatty():
    return False
  try:
    import curses
    curses.setupterm()
    return curses.tigetnum("colors") > 2
  except:
    return False

has_colors = has_colors(sys.stdout)

def printout(text, color=WHITE):
  if has_colors:
    seq = "\x1b[1;%dm" % (30+color) + text + "\x1b[0m"
    return seq
  else:
    return text

class LocalPrinter:
  """
  Print all outputs on standard output, with all the colors and stuff
  """
  def __init__(self, verbosity):
    self.verbosity = verbosity

  def printErrorNoSetFound(self):
    """
    Print 'ErrorNoSetFound' error message
    """
    print printout(settings.strings['errorNoSetFound'], settings.colors['errors'])

  def printIntro(self):
    """
    Print the intro sentence, before testing starts
    """
    print printout(settings.strings['intro'], settings.colors['intro'])

  def printSetIntro(self, u):
    """
    Print the set intro sentence, before the beginning of each test set
    """
    if self.verbosity > 0:
      print printout(u.__class__.__name__ + ': ' + u.__doc__, settings.colors['setIntro'])

  def printTestOutput(self, data, doc):
    """
    Print the output of a test
    """
    if data['success']:
      success = printout(settings.strings['testSuccess'], settings.colors['testSuccess'])
    else:
      success = printout(settings.strings['testFailure'], settings.colors['testFailure'])

    output = settings.strings['testOutputFormat'].format(success=success, return_code=data['code'], elapsed=data['elapsed'], doc=doc)

    if self.verbosity > 1:
      print output

  def printTestDirtyFailure(self, data):
    """
    Print the output of a dirty failed test (aka Exception was thrown during test execution)
    """
    output = printout(settings.strings['testDirtyFailure'], settings.colors['testDirtyFailure']) + str(data['exception'])

    if self.verbosity > 1:
      print output

  def printSetResult(self, test_set, nb_tests, nb_ok, total_response_time):
    """
    Print set results, after the end of each test set
    """
    if self.verbosity > 0:
      percent = int(100 * (float(nb_ok) / float(nb_tests)))
      print printout(
        settings.strings['setResult'].format(nb_tests_passed=nb_ok,
                                             nb_tests_total=nb_tests,
                                             percent=percent,
                                             className=test_set.__class__.__name__),
        settings.colors['setResult'])

  def printTotalResult(self, nb_tests, nb_ok, total_response_time):
    """
    Print total results, after the end of all test sets
    """
    percent = int(100 * (float(nb_ok) / float(nb_tests)))
    print printout(
      settings.strings['totalResult'].format(nb_tests_passed=nb_ok,
                                             nb_tests_total=nb_tests,
                                             percent=percent),
      settings.colors['totalResult'])

    if percent == 100:
      print printout(settings.strings['buildOk'], settings.colors['buildOk'])
    else:
      print printout(settings.strings['buildKo'], settings.colors['buildKo'])



from distutils.core import setup

setup(
  name = 'Aito',
  packages = ['libaito', 'libaito.test_sets'],
  scripts = ['bin/aito'],
  version = '0.5.6',
  description = 'Ultra-lightweight test suite focused on REST API end-to-end tests.',
  author = 'Martin Latrille',
  author_email = 'martinlatrille@live.fr',
  url = 'https://github.com/martinlatrille/Aito', # use the URL to the github repo
  download_url = 'https://github.com/martinlatrille/Aito/tarball/0.5', # I'll explain this in a second
  keywords = ['testing', 'REST', 'api', 'end-to-end'], # arbitrary keywords
  classifiers = [],
  install_requires = [
    "requests >= 2.6.0",
  ],
)



# -*- coding: utf-8 -*-


# Strings of the application
strings = {
  'intro': 'Launching tests\n',
  'setResult': 'Total for {className} : {nb_tests_passed} / {nb_tests_total} successful tests ({percent}%)\n---\n',
  'totalResult': 'Total : {nb_tests_passed} / {nb_tests_total} successful tests ({percent}%)',
  'buildOk': '\n\n  [BUILD: OK]\n\n',
  'buildKo': '\n\n  [BUILD: KO]\n\n',
  'testOutputFormat': '[{success}] [code : {return_code}] [{elapsed}ms] {doc}',
  'testSuccess': 'OK',
  'testFailure': 'KO',
  'testDirtyFailure': '[KO] [DIRTY]',

  'errorNoSetFound': 'No tests found. Exiting.',

}

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

# Colors of the application
colors = {
  'intro': MAGENTA,
  'setIntro': CYAN,
  'setResult': CYAN,
  'totalResult': MAGENTA,
  'buildOk': GREEN,
  'buildKo': RED,
  'testSuccess': GREEN,
  'testFailure': RED,
  'testDirtyFailure': RED,
  'errors': RED,
}



# -*- coding: utf-8 -*-

from core import TestSet

class SetDoesGoogleWork(TestSet):
  """
  Martin wants to ping http://google.com
  """

  def setUp(self):
    self._base_url = 'http://google.com'

  def testPingGoogleHome(self):
    """
    He pings http://google.com
    """
    response = self.get('/')
    return self.expect(response, code=200)

  def testPingGoogleImage(self):
    """
    He pings http://google.com/imghp
    """
    response = self.get('/imghp')
    return self.expect(response, code=200)

  def testPingGoogleNotFound(self):
    """
    He pings http://google.com/imghpqsd
    """
    response = self.get('/imghpqsd')
    return self.expect(response, code=200)

  def testPingGoogleHome2(self):
    """
    He pings http://google.com
    """
    response = self.get('/')
    return self.expect(response, code=200)

class SetDoesStackOverflowWork(TestSet):
  """
  Martin wants to ping http://stackoverflow.com
  """
  _base_url = 'http://stackoverflow.com'

  def testPingStackOvHome(self):
    """
    He pings http://stackoverflow.com
    """
    response = self.get('/')
    return self.expect(response, code=200)


  def testPingStackOvHome2(self):
    """
    He pings http://stackoverflow.com
    """
    response = self.get('/')
    return self.expect(response, code=200)


  def testPingStackOvHome3(self):
    """
    He pings http://stackoverflow.com
    """
    response = self.get('/')
    return self.expect(response, code=200)
